@@ -177,7 +177,7 @@ BOOLEAN egSetScreenSize(IN OUT UINTN *ScreenWidth, IN OUT UINTN *ScreenHeight) {
    EFI_STATUS                            Status = EFI_SUCCESS;
    EFI_GRAPHICS_OUTPUT_MODE_INFORMATION  *Info;
    UINTN                                 Size;
-   UINT32                                ModeNum = 0;
+   UINT32                                ModeNum = 0, CurrentModeNum;
    UINT32                                UGAWidth, UGAHeight, UGADepth, UGARefreshRate;
    BOOLEAN                               ModeSet = FALSE;
 
@@ -185,9 +185,12 @@ BOOLEAN egSetScreenSize(IN OUT UINTN *ScreenWidth, IN OUT UINTN *ScreenHeight) {
       return FALSE;
 
    if (GraphicsOutput != NULL) { // GOP mode (UEFI)
+      CurrentModeNum = GraphicsOutput->Mode->Mode;
       if (*ScreenHeight == 0) { // User specified a mode number (stored in *ScreenWidth); use it directly
          ModeNum = (UINT32) *ScreenWidth;
-         if (egGetResFromMode(ScreenWidth, ScreenHeight) &&
+         if (ModeNum != CurrentModeNum) {
+            ModeSet = TRUE;
+         } else if (egGetResFromMode(ScreenWidth, ScreenHeight) &&
              (refit_call2_wrapper(GraphicsOutput->SetMode, GraphicsOutput, ModeNum) == EFI_SUCCESS)) {
             ModeSet = TRUE;
          }
@@ -199,9 +202,10 @@ BOOLEAN egSetScreenSize(IN OUT UINTN *ScreenWidth, IN OUT UINTN *ScreenHeight) {
          do {
             Status = refit_call4_wrapper(GraphicsOutput->QueryMode, GraphicsOutput, ModeNum, &Size, &Info);
             if ((Status == EFI_SUCCESS) && (Size >= sizeof(*Info) && (Info != NULL)) &&
-                (Info->HorizontalResolution == *ScreenWidth) && (Info->VerticalResolution == *ScreenHeight)) {
-               Status = refit_call2_wrapper(GraphicsOutput->SetMode, GraphicsOutput, ModeNum);
-               ModeSet = (Status == EFI_SUCCESS);
+                (Info->HorizontalResolution == *ScreenWidth) && (Info->VerticalResolution == *ScreenHeight) &&
+                ((ModeNum == CurrentModeNum) ||
+                 (refit_call2_wrapper(GraphicsOutput->SetMode, GraphicsOutput, ModeNum) == EFI_SUCCESS))) {
+               ModeSet = TRUE;
             } // if
          } while ((++ModeNum < GraphicsOutput->Mode->MaxMode) && !ModeSet);
       } // if/else
@@ -217,6 +221,10 @@ BOOLEAN egSetScreenSize(IN OUT UINTN *ScreenWidth, IN OUT UINTN *ScreenHeight) {
             Status = refit_call4_wrapper(GraphicsOutput->QueryMode, GraphicsOutput, ModeNum, &Size, &Info);
             if ((Status == EFI_SUCCESS) && (Info != NULL)) {
                Print(L"Mode %d: %d x %d\n", ModeNum, Info->HorizontalResolution, Info->VerticalResolution);
+               if (ModeNum == CurrentModeNum) {
+                   egScreenWidth = Info->HorizontalResolution;
+                   egScreenHeight = Info->VerticalResolution;
+               } // if
             } // else
          } while (++ModeNum < GraphicsOutput->Mode->MaxMode);
          PauseForKey();
@@ -441,8 +449,9 @@ VOID egDrawImageArea(IN EG_IMAGE *Image,
 // Display a message in the center of the screen, surrounded by a box of the
 // specified color. For the moment, uses graphics calls only. (It still works
 // in text mode on GOP/UEFI systems, but not on UGA/EFI 1.x systems.)
-VOID egDisplayMessage(IN CHAR16 *Text, EG_PIXEL *BGColor) {
+VOID egDisplayMessage(IN CHAR16 *Text, EG_PIXEL *BGColor, UINTN PositionCode) {
    UINTN BoxWidth, BoxHeight;
+   static UINTN Position = 1;
    EG_IMAGE *Box;
 
    if ((Text != NULL) && (BGColor != NULL)) {
@@ -453,20 +462,42 @@ VOID egDisplayMessage(IN CHAR16 *Text, EG_PIXEL *BGColor) {
          BoxWidth = egScreenWidth;
       Box = egCreateFilledImage(BoxWidth, BoxHeight, FALSE, BGColor);
       egRenderText(Text, Box, 7, BoxHeight / 4, (BGColor->r + BGColor->g + BGColor->b) / 3);
-      egDrawImage(Box, (egScreenWidth - BoxWidth) / 2, (egScreenHeight - BoxHeight) / 2);
+      switch (PositionCode) {
+          case CENTER:
+              Position = (egScreenHeight - BoxHeight) / 2;
+              break;
+          case BOTTOM:
+              Position = egScreenHeight - (BoxHeight * 2);
+              break;
+          case TOP:
+              Position = 1;
+              break;
+          default: // NEXTLINE
+              Position += BoxHeight + (BoxHeight / 10);
+              break;
+      } // switch()
+      egDrawImage(Box, (egScreenWidth - BoxWidth) / 2, Position);
+      if ((PositionCode == CENTER) || (Position >= egScreenHeight - (BoxHeight * 5)))
+          Position = 1;
    } // if non-NULL inputs
 } // VOID egDisplayMessage()
 
 // Copy the current contents of the display into an EG_IMAGE....
 // Returns pointer if successful, NULL if not.
 EG_IMAGE * egCopyScreen(VOID) {
+   return egCopyScreenArea(0, 0, egScreenWidth, egScreenHeight);
+} // EG_IMAGE * egCopyScreen()
+
+// Copy the current contents of the specified display area into an EG_IMAGE....
+// Returns pointer if successful, NULL if not.
+EG_IMAGE * egCopyScreenArea(UINTN XPos, UINTN YPos, UINTN Width, UINTN Height) {
    EG_IMAGE *Image = NULL;
 
    if (!egHasGraphics)
       return NULL;
 
-   // allocate a buffer for the whole screen
-   Image = egCreateImage(egScreenWidth, egScreenHeight, FALSE);
+   // allocate a buffer for the screen area
+   Image = egCreateImage(Width, Height, FALSE);
    if (Image == NULL) {
       return NULL;
    }
@@ -474,13 +505,13 @@ EG_IMAGE * egCopyScreen(VOID) {
    // get full screen image
    if (GraphicsOutput != NULL) {
       refit_call10_wrapper(GraphicsOutput->Blt, GraphicsOutput, (EFI_GRAPHICS_OUTPUT_BLT_PIXEL *)Image->PixelData,
-                           EfiBltVideoToBltBuffer, 0, 0, 0, 0, Image->Width, Image->Height, 0);
+                           EfiBltVideoToBltBuffer, XPos, YPos, 0, 0, Image->Width, Image->Height, 0);
    } else if (UgaDraw != NULL) {
       refit_call10_wrapper(UgaDraw->Blt, UgaDraw, (EFI_UGA_PIXEL *)Image->PixelData, EfiUgaVideoToBltBuffer,
-                           0, 0, 0, 0, Image->Width, Image->Height, 0);
+                           XPos, YPos, 0, 0, Image->Width, Image->Height, 0);
    }
    return Image;
-} // EG_IMAGE * egCopyScreen()
+} // EG_IMAGE * egCopyScreenArea()
 
 //
 // Make a screenshot
